@@ -1,37 +1,42 @@
 # -*- coding: utf-8 -*-
 """
-Created on Sun Nov  7 11:58:46 2021
-
+Created on Sat Aug 27 18:11:44 2022
 @author: kentang
 """
 
 import sxtwl, re, math, itertools, datetime, time, ephem
 import numpy as np
 
-#%% 主程式
-class Taiyi:
-    taiyiyear = 10153917
-    year_num = 365
-    threeki_year = 284287
-    jiajishui = 29277
-    circle = 360 #周
-    epoch = 60#紀
-    rounds = 72#局	
-    yuan = 72#元
+def jiazi():
+    Gan = '甲乙丙丁戊己庚辛壬癸'
+    Zhi = '子丑寅卯辰巳午未申酉戌亥'
+    jiazi = [Gan[x % len(Gan)] + Zhi[x % len(Zhi)] for x in range(60)]
+    return jiazi
 
-    def __init__(self, year, month, day, hour):
+class Taiyi():
+    def __init__(self, year, month, day, hour, minute):
         self.year = year
         self.month = month
         self.day = day
         self.hour = hour
-        #干支
-        self.Gan = '甲乙丙丁戊己庚辛壬癸'
-        self.Zhi = '子丑寅卯辰巳午未申酉戌亥'
-        #節氣
+        self.minute = minute
+        self.taiyiyear = 10153917
+        self.year_num = 365
+        self.threeki_year = 284287
+        self.jiajishui = 29277
+        self.circle = 360 #周
+        self.epoch = 60#紀
+        self.rounds = 72#局	
+        self.yuan = 72#元
         self.jieqi = re.findall('..', '春分清明穀雨立夏小滿芒種夏至小暑大暑立秋處暑白露秋分寒露霜降立冬小雪大雪冬至小寒大寒立春雨水驚蟄')
+        self.rujiyuan = (self.taiyiyear + self.year) % 360
+        self.ruyuanzhou = self.rujiyuan // 72
         self.ymc = [11,12,1,2,3,4,5,6,7,8,9,10]
         self.num = [8,3,4,9,2,7,6,1]
         self.rmc = list(range(1,32))
+        #干支
+        self.Gan = '甲乙丙丁戊己庚辛壬癸'
+        self.Zhi = '子丑寅卯辰巳午未申酉戌亥'
         #十六神
         self.sixtengod = dict(zip(list("子丑艮寅卯巽辰巳午未坤申酉戌乾亥"), re.findall("..","地主陽德和德呂申高叢太陽大炅大神大威天道大武武德太簇陰主陰德大義")))
         #間辰
@@ -47,28 +52,29 @@ class Taiyi:
         self.yangji = list("寅丑子亥戌酉申未午巳辰卯")
         self.yingji = list("申未午巳辰卯寅丑子亥戌酉")
         self.ying = {"陰":[4,9,2,7,6,1,8,3], "陽":[1,8,3,4,9,2,7,6]}
-        self.findplace = {"陽":self.yang_sixteen, "陰":self.ying_sixteen}.get(self.kook()[0])
-        self.findplace_num = dict(zip(self.findplace, range(1,19)))
-        self.findji = {"陽":self.yangji, "陰":self.yingji}.get(self.kook()[0])
         #文昌
         self.skyeyes_dict = {"陽" : list("未坤申酉戌乾乾亥子丑艮寅卯巽辰巳午未坤坤申酉戌乾乾亥子丑艮寅卯巽辰巳午未坤坤申酉戌乾乾亥子丑艮寅卯巽辰巳午未坤坤申酉戌乾乾亥子丑艮寅卯巽辰巳午未"),
         "陰":list("寅卯辰巽巽巳午未坤申酉戌乾亥子丑艮艮寅卯辰巽巽巳午未坤申酉戌乾亥子丑艮艮寅卯辰巽巽巳午未坤申酉戌乾亥子丑艮艮寅卯辰巽巽巳午未坤申酉戌乾亥子丑艮艮")}
-        self.skyeyes = dict(zip(range(1,73),self.skyeyes_dict.get(self.kook()[0]))).get(int(self.kook().replace("陰遁", "").replace("陽遁", "").replace("局", "")))
+        
         #太歲
         self.taishui = self.gangzhi()[3][1]
         #合神
         self.hegod = dict(zip(list("子寅卯辰巳午丑亥戌酉申未"),list("丑亥戌酉申未子寅卯辰巳午"))).get(self.taishui)
-        #計神
-        self.jigod = dict(zip(self.Zhi, self.findji )).get(self.taishui)
         #八門
         self.ed = list("開休生傷杜景死驚")
         self.dz_date = datetime.datetime.strptime(str(self.year)+"/"+str(self.month)+"/"+str(self.day) , '%Y/%m/%d') - datetime.timedelta(days=self.dzdistance())
-        self.accHour = int(int((29277 + self.dz_date.year) * 365.2425) * 12) + ((self.dzdistance()-1) * 12) + dict(zip(self.Zhi,range(1,13))).get(self.gangzhi()[3][1])
-       
-    def jiazi(self):
-        jiazi = [self.Gan[x % len(self.Gan)] + self.Zhi[x % len(self.Zhi)] for x in range(60)]
-        return jiazi
-       
+
+    def skyeyes(self, ji):
+        findplace = {"陽":self.yang_sixteen, "陰":self.ying_sixteen}.get(self.kook(ji)[0])
+        findplace_num = dict(zip(findplace, range(1,19)))    
+        return dict(zip(range(1,73),self.skyeyes_dict.get(self.kook(ji)[0]))).get(int(self.kook(ji).replace("陰遁", "").replace("陽遁", "").replace("局", "")))
+
+    #計神
+    def jigod(self, ji):
+        findji = {"陽":self.yangji, "陰":self.yingji}.get(self.kook(ji)[0])
+        return dict(zip(self.Zhi, findji )).get(self.taishui)
+
+
     def new_list(self, olist, o):
         zhihead_code = olist.index(o)
         res1 = []
@@ -91,14 +97,10 @@ class Taiyi:
     #節氣
     def ecliptic_lon(self, jd_utc):
         s = ephem.Sun(jd_utc)
-        equ = ephem.Equatorial(s.ra,s.dec,epoch=jd_utc)
-        e= ephem.Ecliptic(equ)
-        return e.lon
+        return ephem.Ecliptic(ephem.Equatorial(s.ra,s.dec,epoch=jd_utc)).lon
     
     def sta(self, jd):
-        e=self.ecliptic_lon(jd)
-        n=int(e*180.0/math.pi/15)
-        return n
+        return int(self.ecliptic_lon(jd)*180.0/math.pi/15)
     
     def iteration(self, jd):
         s1=self.sta(jd)
@@ -117,8 +119,7 @@ class Taiyi:
     def fjqs(self, year, month, day, hour):
         jd = ephem.Date( str(year)+"/"+str(month).zfill(2)+"/"+str(day).zfill(2)+" "+str(hour).zfill(2)+":00:00.00")
         ct = datetime.datetime.strptime(str(year)+"-"+str(month)+"-"+str(day)+"-"+str(hour)+":00:00", "%Y-%m-%d-%H:%M:%S")
-        e=self.ecliptic_lon(jd)
-        n=int(e*180.0/math.pi/15)+1
+        n=int(self.ecliptic_lon(jd)*180.0/math.pi/15)+1
         c = []
         for i in range(1):
             if n>=24:
@@ -128,20 +129,6 @@ class Taiyi:
             b = [self.jieqi[n], datetime.datetime.strptime(str(d[0])+"-"+str(d[1])+"-"+str(d[2])+"-"+str(d[3])+":00:00", "%Y-%m-%d-%H:%M:%S")]
             c.append(b)
         return c[0]
-    
-    def jq(self, year, month, day, hour):
-        jd= ephem.Date( str(year)+"/"+str(month).zfill(2)+"/"+str(day).zfill(2)+" "+str(hour).zfill(2)+":00:00.00")
-        ct = datetime.datetime.strptime(str(year)+"-"+str(month)+"-"+str(day)+"-"+str(hour)+":00:00", "%Y-%m-%d-%H:%M:%S")
-        p = ct - datetime.timedelta(days=7)
-        pp = ct - datetime.timedelta(days=21)
-        bf = self.fjqs(p.year, p.month, p.day, p.hour)
-        bbf = self.fjqs(pp.year, pp.month, pp.day, pp.hour)
-        t1 = bf[1]
-        t2 = bbf[1]
-        if ct < t1:
-            return bbf[0]
-        else:
-            return bf[0]
 
     def find_jq_date(self, year, month, day, hour, jq):
         jd=ephem.Date( str(year)+"/"+str(month).zfill(2)+"/"+str(day).zfill(2)+" "+str(hour).zfill(2)+":00:00.00")
@@ -169,45 +156,56 @@ class Taiyi:
     def xzdistance(self):
         return [self.find_jq_date(self.year, self.month, self.day, self.hour, "夏至") -  datetime.datetime(self.year,self.month, self.day, self.hour, 0,0)][0].days           
             
-    def kook(self):
+    def accnum(self, ji):
+        if ji == 0: #年計
+            if self.year >= 0:
+                return self.taiyiyear + self.year 
+            elif self.year < 0:
+                return self.taiyiyear + self.year + 1 
+        elif ji == 1: #月計
+            if self.year >= 0:
+                accyear = self.taiyiyear + self.year - 1
+            elif self.year < 0:
+                accyear = self.taiyiyear + self.year + 1 
+            return accyear * 12 + 2 + self.month
+        elif ji == 2:#日計
+            return (datetime.datetime.strptime(
+            "{0:04}-{1:02d}-{2:02d} 00:00:00".format(self.year, self.month, self.day),
+            "%Y-%m-%d %H:%M:%S") - datetime.datetime.strptime("1900-06-19 00:00:00",
+                                            "%Y-%m-%d %H:%M:%S")).days
+        elif ji == 3: #時計
+            return ((datetime.datetime.strptime(
+            "{0:04}-{1:02d}-{2:02d} 00:00:00".format(self.year, self.month, self.day),
+            "%Y-%m-%d %H:%M:%S") - datetime.datetime.strptime("1900-06-19 00:00:00",
+                                            "%Y-%m-%d %H:%M:%S")).days - 1 ) * 12 + (self.hour + 1 ) // 2 + 1
+    def kook(self, ji):
         dz = self.dzdistance()
         xz = self.xzdistance()
         dz_date = datetime.datetime.strptime(str(self.year)+"/"+str(self.month)+"/"+str(self.day) , '%Y/%m/%d') - datetime.timedelta(days=dz)
         xz_date = datetime.datetime.strptime(str(self.year)+"/"+str(self.month)+"/"+str(self.day) , '%Y/%m/%d') - datetime.timedelta(days=xz)
         current_date = datetime.datetime.strptime(str(self.year)+"/"+str(self.month)+"/"+str(self.day) , '%Y/%m/%d')
-        accHour = int((29277 + dz_date.year) * 365.2425) * 12 
-        dz_kook = accHour % 360 % 72
-        if current_date >= xz_date and self.month >= 6:
-            dun = "陰遁"
-        else:
+        if ji == 0 or ji == 1 or ji ==2:
             dun = "陽遁"
-        if self.hour == 23 and dz_kook + (dz * 12) % 72 == 60:
-            kook = 1
-            return dun + str(kook) + "局"
-        elif self.hour == 23 and dz_kook + (dz * 12) % 72 != 60:
-            kook = dz_kook + (dz * 12) % 72 + dict(zip(self.Zhi,range(1,13))).get(self.gangzhi()[3][1])
-            dunkook = kook-60
-            if dunkook < 0:
-                dunkook = kook + 12
-            return dun + str(dunkook) + "局"
-        elif self.hour != 23:
-            kook = dz_kook + (dz * 12) % 72 + dict(zip(self.Zhi,range(1,13))).get(self.gangzhi()[3][1])
-            if kook > 72:
-                kook = kook - 72
-            return dun + str(kook) + "局"
-
-    def getyuan(self):
-        accnum = self.year + self.taiyiyear
+            return dun + str(self.accnum(ji)%72) + "局"
+        elif ji == 3:
+            if current_date >= xz_date and self.month >= 6:
+                dun = "陰遁"
+            else:
+                dun = "陽遁"
+            return dun + str(self.accnum(ji)%72) + "局"
+    
+    def getyuan(self, ji):
+        accnum = self.accnum(ji)
         if round(accnum % self.circle) == 1:
             find_ji_num = 1
         else:
             find_ji_num = int(round((accnum % self.circle) / self.yuan, 0))
-        fiveyuen_d = dict(zip(range(1,6), [self.jiazi()[i] for i in [0,12,24,36,48]]))
+        fiveyuen_d = dict(zip(range(1,6), jiazi()[0::12]))
         jiyuan = fiveyuen_d.get(find_ji_num) 
         return jiyuan
     
-    def getepoch(self):
-        accnum = self.year + self.taiyiyear
+    def getepoch(self, ji):
+        accnum = self.accnum(ji)
         if round(accnum % self.circle) == 1:
             find_ji_num = 1
         else:
@@ -215,8 +213,8 @@ class Taiyi:
         cnum = list("一二三四五六七八九十")
         return "第"+dict(zip(range(1,8), cnum[0:7])).get(find_ji_num)+"紀"
     
-    def jiyuan(self):
-        return self.getyuan()+ "元" + self.getepoch()
+    def jiyuan(self, ji):
+        return self.getyuan(ji)+ "元" + self.getepoch(ji)
     
     def tdate(self):
         return dict(zip(list("年月日時"), [self.taiyiyear + self.year, 
@@ -254,18 +252,18 @@ class Taiyi:
         #四郭杜 : 文昌與客參將同宮，客大，主參將同宮，又逢掩、迫、關、囚
         return 
 
-    def ty(self):
+    def ty(self, ji):
         arr = np.arange(10) 
         repetitions = 3
         arrangement = np.repeat(arr, repetitions) 
         arrangement_r = list(reversed(arrangement))
         yy_dict = {"陽": dict(zip(range(1,73), list(itertools.chain.from_iterable([list(arrangement)[3:15]+ list(arrangement)[18:]] * 3)))),  "陰": dict(zip(range(1,73), (list(arrangement_r)[:12] + list(arrangement_r)[15:][:-3]) * 3))}
-        return yy_dict.get(self.kook()[0]).get(int(self.kook().replace("陰遁", "").replace("陽遁", "").replace("局", "")))   
+        return yy_dict.get(self.kook(ji)[0]).get(int(self.kook(ji).replace("陰遁", "").replace("陽遁", "").replace("局", "")))   
     
     #始擊
-    def sf(self):
-        wc = self.skyeyes
-        jd = self.jigod
+    def sf(self, ji):
+        wc = self.skyeyes(ji)
+        jd = self.jigod(ji)
         if wc == jd:
             sf = "艮"
             return sf
@@ -276,8 +274,8 @@ class Taiyi:
             return  start2
         
     #定目
-    def se(self):
-        wc = self.skyeyes
+    def se(self, ji):
+        wc = self.skyeyes(ji)
         hg = self.hegod
         ts = self.taishui
         start = self.new_list(self.gong1, hg)
@@ -285,14 +283,14 @@ class Taiyi:
         start2 = self.new_list(self.gong1, wc)[start1-1]
         return  start2
 
-    def home_cal(self):
+    def home_cal(self, ji):
         num = self.num
         lnum = [8, 8, 3,  3, 4,4, 9, 9, 2, 2, 7, 7, 6, 6, 1, 1]
         gong = list("亥子丑艮寅卯辰巽巳午未坤申酉戌乾")
-        wc = self.skyeyes
+        wc = self.skyeyes(ji)
         lg = dict(zip(gong, lnum))
         wc_num = lg.get(wc)
-        ty = self.ty()
+        ty = self.ty(ji)
         wc_jc =  [i == wc for i in self.jc].count(True)
         ty_jc = [i == ty for i in self.tyjc].count(True)
         wc_jc1 =  [i == wc for i in self.jc1].count(True)
@@ -312,26 +310,26 @@ class Taiyi:
         elif wc_jc !=1 and ty_jc !=1 and wc_jc1 !=1 and ty == wc_num:
             return ty
         
-    def home_general(self):
-        home_g = self.home_cal()  % 10
+    def home_general(self, ji):
+        home_g = self.home_cal(ji)  % 10
         if home_g == 0:
             home_g = 5
         return home_g
     
-    def home_vgen(self):
-        home_vg = self.home_general() *3 % 10
+    def home_vgen(self, ji):
+        home_vg = self.home_general(ji) *3 % 10
         if home_vg ==0:
             home_vg = 5
         return home_vg
         
-    def away_cal(self):
+    def away_cal(self, ji):
         num = self.num
         lnum = [8, 8, 3,  3, 4,4, 9, 9, 2, 2, 7, 7, 6, 6, 1, 1]
         gong = list("亥子丑艮寅卯辰巽巳午未坤申酉戌乾")
-        sf = self.sf()
+        sf = self.sf(ji)
         lg = dict(zip(gong, lnum))
         sf_num = lg.get(sf)
-        ty = self.ty()
+        ty = self.ty(ji)
         sf_jc =  [i == sf for i in self.jc].count(True)
         ty_jc = [i == ty for i in self.tyjc].count(True)
         sf_jc1 =  [i == sf for i in self.jc1].count(True)
@@ -351,26 +349,26 @@ class Taiyi:
         elif sf_jc !=1 and ty_jc !=1 and sf_jc1 !=1 and sf_num == ty:
             return ty
     
-    def away_general(self):
-        away_g = self.away_cal()  % 10
+    def away_general(self, ji):
+        away_g = self.away_cal(ji)  % 10
         if away_g == 0:
             away_g = 5
         return away_g
     
-    def away_vgen(self):
-        away_vg = self.away_general() *3 % 10
+    def away_vgen(self, ji):
+        away_vg = self.away_general(ji) *3 % 10
         if away_vg == 0:
             away_vg = 5
         return away_vg
         
-    def set_cal(self):
+    def set_cal(self, ji):
         num = self.num
         lnum = [8, 8, 3,  3, 4,4, 9, 9, 2, 2, 7, 7, 6, 6, 1, 1]
         gong = list("亥子丑艮寅卯辰巽巳午未坤申酉戌乾")
-        se = self.se()
+        se = self.se(ji)
         lg = dict(zip(gong, lnum))
         se_num = lg.get(se)
-        ty = self.ty()
+        ty = self.ty(ji)
         se_jc =  [i == se for i in self.jc].count(True)
         ty_jc = [i == ty for i in self.tyjc].count(True)
         se_jc1 =  [i == se for i in self.jc1].count(True)
@@ -388,7 +386,7 @@ class Taiyi:
         elif se_jc !=1 and ty_jc !=1 and se_jc1 !=1 :
             return sum(se_order[: se_order.index(ty)])
         elif se_jc !=1 and ty_jc !=1 and se_jc1 !=1 and se_num != ty:
-            return sum(sf_order[: sf_order.index(ty)])
+            return sum(se_order[: se_order.index(ty)])
         elif se_jc !=1 and ty_jc !=1 and se_jc1 !=1 and se_num == ty:
             return ty
     
@@ -409,23 +407,23 @@ class Taiyi:
         t.append(numdict.get(num, None))
         return [i for i in t if i is not None]
     
-    def set_general(self):
-        set_g = self.set_cal()  % 10
+    def set_general(self, ji):
+        set_g = self.set_cal(ji)  % 10
         if set_g == 0:
             set_g = 5
         return set_g
     
-    def set_vgen(self):
-        set_vg =  self.set_general() *3 % 10
+    def set_vgen(self, ji):
+        set_vg =  self.set_general(ji) *3 % 10
         if set_vg == 0:
             set_vg = 5
         return set_vg
     
-    def sixteen_gong(self):
-        dict1 = [{self.skyeyes:"文昌"},{self.taishui:"太歲"},{self.hegod:"合神"},{self.sf():"始擊"},
-                 {self.se():"定目"}, {self.kingbase():"君基"}, {self.officerbase():"臣基"}, {self.pplbase():"民基"},
-                 {self.fgd():"四神"},{self.skyyi():"天乙"},{self.earthyi():"地乙"},{self.zhifu():"直符"},
-                 {self.flyfu():"飛符"},{self.kingfu():"帝符"},{self.taijun():"太尊"}, {self.wufu():"五福"} ]
+    def sixteen_gong(self, ji):
+        dict1 = [{self.skyeyes(ji):"文昌"},{self.taishui:"太歲"},{self.hegod:"合神"},{self.sf(ji):"始擊"},
+                 {self.se(ji):"定目"}, {self.kingbase(ji):"君基"}, {self.officerbase(ji):"臣基"}, {self.pplbase(ji):"民基"},
+                 {self.fgd(ji):"四神"},{self.skyyi(ji):"天乙"},{self.earthyi(ji):"地乙"},{self.zhifu(ji):"直符"},
+                 {self.flyfu(ji):"飛符"},{self.kingfu(ji):"帝符"},{self.taijun(ji):"太尊"}, {self.wufu(ji):"五福"} ]
         res = {"子":"", "丑":"", "艮":"","寅":"", "卯":"", "辰":"", "巽":"","巳":"", "午":"", "未":"", "申":"", "坤":"", "酉":"", "戌":"", "乾":"", "亥":"", "中":""}
         for dict in dict1:
             for list in dict:
@@ -444,12 +442,12 @@ class Taiyi:
         r = str(res.keys())[11:].replace("([","").replace("'","").replace("])","").replace(" ", "").split(",")
         return {r[i]:rrres[i] for i in range(0,16)}
 
-    def nine_gong(self):
-        dict1 = [{self.home_general():"主將"},{self.home_vgen():"主參"},{self.away_general():"客將"},
-                 {self.away_vgen():"客參"},{self.set_general():"定將"},{self.set_vgen():"定參"},
-                 {self.ty():"太乙"}, {self.threewind():"三風"},  {self.fivewind():"五風"},
-                 {self.eightwind():"八風"},  {self.flybird():"飛鳥"},{self.bigyo():"大游"},
-                 {self.smyo():"小游"},]
+    def nine_gong(self, ji):
+        dict1 = [{self.home_general(ji):"主將"},{self.home_vgen(ji):"主參"},{self.away_general(ji):"客將"},
+                 {self.away_vgen(ji):"客參"},{self.set_general(ji):"定將"},{self.set_vgen(ji):"定參"},
+                 {self.ty(ji):"太乙"}, {self.threewind(ji):"三風"},  {self.fivewind(ji):"五風"},
+                 {self.eightwind(ji):"八風"},  {self.flybird(ji):"飛鳥"},{self.bigyo(ji):"大游"},
+                 {self.smyo(ji):"小游"},]
         res = {8:"", 3:"", 4:"", 9:"",5:"", 2:"", 7:"", 6:"", 1:""}
         for dict in dict1:
             for list in dict:
@@ -466,30 +464,30 @@ class Taiyi:
         return res 
     
     #君基
-    def kingbase(self):
-        kb = (self.accHour +250) % 360  / 30
+    def kingbase(self, ji):
+        kb = (self.accnum(ji) +250) % 360  / 30
         kb_v = dict(zip(range(1,13), self.new_list(self.Zhi, "午"))).get(int(kb))
         if kb_v == 0 or kb_v ==None:
             kb_v = "中"
         return kb_v
     
     #臣基
-    def officerbase(self):
-        ob = (self.accHour +250) % 360  % 36 / 3
+    def officerbase(self, ji):
+        ob = (self.accnum(ji)  +250) % 360  % 36 / 3
         ob_v =  dict(zip(range(1,13), self.new_list(self.Zhi, "午"))).get(int(ob))
         if ob_v == 0 or ob_v ==None:
             ob_v = "中"
         return ob_v
     #民基
-    def pplbase(self):
-        pb = (self.accHour +250) % 360 % 12
+    def pplbase(self, ji):
+        pb = (self.accnum(ji)  +250) % 360 % 12
         pb_v = dict(zip(range(1,13), self.new_list(self.Zhi, "戌"))).get(int(pb))
         if pb_v == 0 or pb_v ==None:
             pb_v = "中"
         return  pb_v
     #大游
-    def bigyo(self):
-        by = int((self.accHour +34) % 388)
+    def bigyo(self, ji):
+        by = int((self.accnum(ji) +34) % 388)
         if by < 36:
             by = by
         elif by > 36:
@@ -499,8 +497,8 @@ class Taiyi:
             byv = 5
         return byv
     #小游
-    def smyo(self):
-        sy = int(self.accHour  % 360)
+    def smyo(self, ji):
+        sy = int(self.accnum(ji)  % 360)
         if sy < 24:
             sy = sy % 3 
         elif sy > 24:
@@ -512,43 +510,43 @@ class Taiyi:
             syv = 5
         return syv
     #四神
-    def fgd(self):
-        f = self.accHour % 360 % 36 / 3 
+    def fgd(self, ji):
+        f = self.accnum(ji)  % 360 % 36 / 3 
         fv = dict(zip(range(1,13), self.new_list(self.Zhi, "亥"))).get(int(f))
         if fv == 0 or fv == None:
             fv = "中"
         return fv
     #天乙
-    def skyyi(self):
-        f = self.accHour % 360 % 36 / 3 
+    def skyyi(self, ji):
+        f = self.accnum(ji)  % 360 % 36 / 3 
         fv = dict(zip(range(1,13), self.new_list(self.Zhi, "酉"))).get(int(f))
         if fv == 0 or fv == None:
             fv = "中"
         return fv
     #地乙
-    def earthyi(self):
-        f = self.accHour % 360 % 36 / 3
+    def earthyi(self, ji):
+        f = self.accnum(ji)  % 360 % 36 / 3
         fv = dict(zip(range(1,13), self.new_list(self.Zhi, "巳"))).get(int(f))
         if fv == 0 or fv == None:
             fv = "中"
         return fv
     #直符
-    def zhifu(self):
-        f = self.accHour % 360 % 36 / 3
+    def zhifu(self, ji):
+        f = self.accnum(ji)  % 360 % 36 / 3
         fv = dict(zip(range(1,14), ["中"]+self.new_list(self.Zhi, "酉"))).get(int(f))
         if fv == 0 or fv == None:
             fv = "中"
         return fv
     #飛符
-    def flyfu(self):
-        f = self.accHour % 360 % 36 / 3
+    def flyfu(self, ji):
+        f = self.accnum(ji) % 360 % 36 / 3
         fv = dict(zip(range(1,13), self.new_list(self.Zhi, "辰"))).get(int(f))
         if fv == 0 or fv == None:
             fv = "中"
         return fv
     #帝符
-    def kingfu(self):
-        f = self.accHour  %20
+    def kingfu(self, ji):
+        f = self.accnum(ji)  %20
         if f > 16:
             f = f - 16
         fv = dict(zip(range(1,17), self.new_list(self.gong1, "戌"))).get(int(f))
@@ -556,37 +554,37 @@ class Taiyi:
             fv = "中"
         return fv
     #太尊
-    def taijun(self):
-        f = self.accHour  % 4
+    def taijun(self, ji):
+        f = self.accnum(ji)   % 4
         fv = dict(zip(range(1,5), list("子午卯酉"))).get(int(f))
         if fv == 0  or fv == None:
             fv = "中"
         return fv
     #飛鳥
-    def flybird(self):
-        f = self.accHour  % 9
+    def flybird(self, ji):
+        f = self.accnum(ji)   % 9
         fv = dict(zip(range(1,10), [1,8,3,4,9,2,7,6])).get(int(f))
         if fv == 0 or fv ==None:
             fv = 5
         return fv
     #五行
-    def wuxing(self):
-        f = int(self.accHour) // 5
+    def wuxing(self, ji):
+        f = self.accnum(ji) // 5
         f = f % 5
         fv =  dict(zip(range(1,10), [1,3,5,7,9,2,4,6,8])).get(int(f))
         if fv == 0 or fv ==None:
             fv = 5
         return fv
     #三風
-    def threewind(self):
-        f = int(self.accHour) % 9
+    def threewind(self, ji):
+        f = self.accnum(ji)  % 9
         fv = dict(zip(range(1,9), [7,2,6,1,5,9,4,8])).get(int(f))
         if fv == 0 or fv == None:
             fv = 5
         return fv
     #五風
-    def fivewind(self):
-        f = int(self.accHour) % 29
+    def fivewind(self, ji):
+        f = self.accnum(ji)  % 29
         if f > 10:
             f = f - 9
         fv = dict(zip(range(1,10), [1,3,5,7,9,2,4,6,8])).get(int(f))
@@ -594,22 +592,22 @@ class Taiyi:
             fv = 5
         return fv
     #八風
-    def eightwind(self):
-        f = int(self.accHour) % 9
+    def eightwind(self, ji):
+        f = self.accnum(ji)  % 9
         fv = dict(zip(range(1,9), [2,3,5,6,7,8,9,1])).get(int(f))
         if fv == 0 or fv == None:
             fv = 5
         return fv
     #五福
-    def wufu(self):
-        f = int(self.accHour + 250) % 225 / 45 
+    def wufu(self, ji):
+        f = int(self.accnum(ji)  + 250) % 225 / 45 
         fv = dict(zip(range(1,6), list("乾艮巽坤中"))).get(int(f))
         if fv == 0 or fv ==None:
             fv = 5
         return fv
     #八門
-    def eight_door(self):
-        acc = self.accHour % 120
+    def eight_door(self, ji):
+        acc = self.accnum(ji) % 120
         if acc == 0:
             acc = 120
         eightdoor_zhishi = acc // 30
@@ -619,130 +617,131 @@ class Taiyi:
             fdoor = [1,8,3,6]
         elif self.kook()[0] == "陰":
             fdoor = [9,2,7,4]
-        ty_gong = self.ty()
+        #ty_gong = self.ty()
         return self.new_list(self.num, fdoor[eightdoor_zhishi]), eightdoor_zhishi
     
-    def pan(self):
+    def pan(self, ji):
         return {
+                "太乙計":{0:"年計", 1:"月計", 2:"日計", 3:"時計"}.get(ji), 
                 "干支":self.gangzhi(),
                 "農曆":self.lunar_date_d(),
-                "紀元":self.jiyuan(),
-                "時局":self.kook(),
-                "太乙":self.ty(),
-                "文昌":self.skyeyes,
+                "紀元":self.jiyuan(ji),
+                "時局":self.kook(ji),
+                "太乙":self.ty(ji),
+                "文昌":self.skyeyes(ji),
                 "太歲":self.taishui,
                 "合神":self.hegod,
-                "計神":self.jigod,
-                "始擊":self.sf(),
-                "定目":self.se(),
-                "主算":[self.home_cal(), self.cal_des(self.home_cal())],
-                "客算":[self.away_cal(), self.cal_des(self.away_cal())],
-                "定算":[self.set_cal(), self.cal_des(self.set_cal())],
-                "九宮":self.nine_gong(), 
-                "十六宮":self.sixteen_gong(),
+                "計神":self.jigod(ji),
+                "始擊":self.sf(ji),
+                "定目":self.se(ji),
+                "主算":[self.home_cal(ji), self.cal_des(self.home_cal(ji))],
+                "客算":[self.away_cal(ji), self.cal_des(self.away_cal(ji))],
+                "定算":[self.set_cal(ji), self.cal_des(self.set_cal(ji))],
+                "九宮":self.nine_gong(ji), 
+                "十六宮":self.sixteen_gong(ji),
                 }
     
-    def html(self):
+    def html(self, ji):
         text = '''<html><body><table border="0" cellpadding="1" cellspacing="1" style="width:500px">
     	<tbody>
     		<tr>
     			<td colspan="5">
     			<p><span style="font-size:large"><strong>'''+str(self.year)+"年"+str(self.month)+"月"+str(self.day)+"日"+str(self.hour)+"時"+'''<br />
     			干支: '''+self.gangzhi()[0]+"  "+self.gangzhi()[1]+"  "+self.gangzhi()[2]+"  "+self.gangzhi()[3]+'''&nbsp;</strong></span></p>
-    			<p><span style="font-size:large"><strong>'''+self.jiyuan()+"  "+self.kook()+'''<br />
-    			主算:'''+str(self.home_cal())+"".join(self.cal_des(self.home_cal()))+'''<br />
-    			客算:'''+str(self.away_cal())+"".join(self.cal_des(self.away_cal()))+'''<br />
-    			定算:'''+str(self.set_cal())+"".join(self.cal_des(self.set_cal()))+'''</strong></span></p>
+    			<p><span style="font-size:large"><strong>'''+self.jiyuan(ji)+"  "+self.kook(ji)+'''<br />
+    			主算:'''+str(self.home_cal(ji))+"".join(self.cal_des(self.home_cal(ji)))+'''<br />
+    			客算:'''+str(self.away_cal(ji))+"".join(self.cal_des(self.away_cal(ji)))+'''<br />
+    			定算:'''+str(self.set_cal(ji))+"".join(self.cal_des(self.set_cal(ji)))+'''</strong></span></p>
     			</td>
     		</tr>
     		<tr>
     			<td><span style="font-size:large"><strong>巽</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("巽")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("巽")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>巳</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("巳")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("巳")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>午</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("午")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("午")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>未&nbsp;</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("未")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("未")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>坤</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("坤")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("坤")])+'''</tbody>
     			</table>
     			</td>
     		</tr>
     		<tr>
     			<td><span style="font-size:large"><strong>辰</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("辰")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("辰")])+'''</tbody>
     			</table>
     			</td>
     			<td colspan="3" rowspan="3">&nbsp;</td>
     			<td><span style="font-size:large"><strong>申</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("申")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("申")])+'''</tbody>
     			</table>
     			</td>
     		</tr>
     		<tr>
     			<td><span style="font-size:large"><strong>卯</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("卯")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("卯")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>酉</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("酉")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("酉")])+'''</tbody>
     			</table>
     			</td>
     		</tr>
     		<tr>
     			<td><span style="font-size:large"><strong>寅</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("寅")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("寅")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>戌</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("戌")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("戌")])+'''</tbody>
     			</table>
     			</td>
     		</tr>
     		<tr>
     			<td><span style="font-size:large"><strong>艮</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("艮")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("艮")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>丑</strong></span>
     				<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("丑")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("丑")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>子</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("子")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("子")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>亥</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("亥")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("亥")])+'''</tbody>
     			</table>
     			</td>
     			<td><span style="font-size:large"><strong>乾</strong></span>
     			<table border="0" cellpadding="1" cellspacing="1" style="width:100px">
-    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan().get("十六宮").get("乾")])+'''</tbody>
+    				<tbody>'''+"".join(['''<tr><td>'''+i+'''</td></tr>''' for i in self.pan(ji).get("十六宮").get("乾")])+'''</tbody>
     			</table>
     			</td>
     		</tr>
@@ -750,13 +749,14 @@ class Taiyi:
     </table>
     
     <p>&nbsp;</p></body></html>
-
          ''' 
         return text
 
 
+
 if __name__ == '__main__':
     tic = time.perf_counter()
-    print(Taiyi(2015,7,20,10).pan())
+    print(Taiyi(2022,8,27,18,14).pan(3))
     toc = time.perf_counter()
-    print(f"{toc - tic:0.4f} seconds")
\ No newline at end of file
+    print(f"{toc - tic:0.4f} seconds")
+    
\ No newline at end of file
